__all__ = ['IFirmware', 'IFirmware2', 'IUnit']


class IDevice:
    def __init__(self):
        pass


class IFirmware:
    def __init__(self):
        pass

    @property
    def version(self):
        """
        :rtype: str
        """
        raise NotImplementedError

    def write_rw_bit(self, address, int1_value, ack_tout_sec):
        """
        :type address: int
        :type int1_value: bool | int
        :type ack_tout_sec: float

        :raises PsuCtlError:
        :return: int1_value
        :rtype: int
        """
        raise NotImplementedError

    def write_rw_reg(self, address, int16_value, ack_tout_sec):
        """
        :type address: int
        :type int16_value: int
        :type ack_tout_sec: float

        :raises PsuCtlError:
        :return: int16_value
        :rtype: int
        """
        raise NotImplementedError


class IFirmware2:
    STAT_CHAN_STEPS = '1'

    CAP_CHAN_MAX_STEPS = '1'
    CAP_CHAN_STEP_TOUT = '2'

    def __init__(self):
        pass

    def write(self, query_msg):
        """
        :type query_msg: str
        :rtype: int
        """
        raise NotImplementedError

    def read(self):
        """
        :return: reply_msg
        :rtype: str
        """
        raise NotImplementedError

    def get_chan_cap(self, chan_ref, cap_name):
        """
        :type chan_ref: str
        :type cap_name: str
        :rtype: int
        """
        raise NotImplementedError

    def get_chan_stat(self, chan_ref, stat_name):
        """
        :type chan_ref: str
        :type stat_name: str
        :rtype: int
        """
        raise NotImplementedError


class IUnit:
    def __init__(self, firmware, ref):
        """
        :type firmware: IFirmware| IFirmware2
        :type ref: list | tuple
        """
        self._firmware = firmware
        self._ref = ref

    def run(self, conf):
        """
        :type conf: dict
        :rtype: None
        """
        raise NotImplementedError
